@@ -44,8 +44,6 @@
 // config file
 require_once("backend/vcarddir/config.php");
 
-include_once('lib/default/diffbackend/diffbackend.php');
-
 class BackendVCardDir extends BackendDiff {
     /**----------------------------------------------------------------------------------------------------------
      * default backend methods
@@ -678,4 +676,3 @@ class BackendVCardDir extends BackendDiff {
         return $data;
     }
 };
-?>
\ No newline at end of file
