@@ -55,8 +55,8 @@ class ZPushException extends Exception {
         if (!$logLevel)
             $logLevel = $this->defaultLogLevel;
 
-        ZLog::Write($logLevel, get_class($this) .': '. $message . ' - code: '.$code, false);
         parent::__construct($message, (int) $code);
+        ZLog::Write($logLevel, get_class($this) .': '. $message . ' - code: '.$code. ' - file: '. $this->getFile().':'.$this->getLine(), false);
     }
 
     public function getHTTPCodeString() {
@@ -71,4 +71,3 @@ class ZPushException extends Exception {
         return $this->showLegal;
     }
 }
-?>
\ No newline at end of file
