@@ -83,10 +83,6 @@ abstract class RequestProcessor {
 
         // mark this request as "authenticated"
         self::$userIsAuthenticated = true;
-
-        // check Auth-User's permissions on GETUser's store
-        if($backend->Setup(Request::GetGETUser(), true) == false)
-            throw new AuthenticationRequiredException(sprintf("Not enough privileges of '%s' to setup for user '%s': Permission denied", Request::GetAuthUser(), Request::GetGETUser()));
     }
 
     /**
@@ -127,8 +123,21 @@ abstract class RequestProcessor {
     static public function HandleRequest() {
         $handler = ZPush::GetRequestHandlerForCommand(Request::GetCommandCode());
 
-        // TODO handle WBXML exceptions here and print stack
-        return $handler->Handle(Request::GetCommandCode());
+        // if there is an error decoding wbxml, consume remaining data and include it in the WBXMLException
+        try {
+            if (!$handler->Handle(Request::GetCommandCode())) {
+                throw new WBXMLException(sprintf("Unknown error in %s->Handle()", get_class($handler)));
+            }
+        }
+        catch (Exception $ex) {
+            ZLog::Write(LOGLEVEL_FATAL, "WBXML debug data: " . Request::GetInputAsBase64(), false);
+            throw $ex;
+        }
+
+        // also log WBXML in happy case
+        if (@constant('WBXML_DEBUG') === true) {
+            ZLog::Write(LOGLEVEL_WBXML, "WBXML-IN : ". Request::GetInputAsBase64(), false);
+        }
     }
 
     /**
@@ -154,4 +163,3 @@ abstract class RequestProcessor {
      */
     abstract public function Handle($commandCode);
 }
-?>
\ No newline at end of file
