@@ -6,7 +6,7 @@
 *
 * Created   :   16.02.2012
 *
-* Copyright 2007 - 2013 Zarafa Deutschland GmbH
+* Copyright 2007 - 2016 Zarafa Deutschland GmbH
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
@@ -55,6 +55,24 @@ class Settings extends RequestProcessor {
         if (!self::$decoder->getElementStartTag(SYNC_SETTINGS_SETTINGS))
             return false;
 
+        // add capability header for KOE
+        if(self::$deviceManager->IsKoe()) {
+            // define the supported capabilites
+            $cap = array();
+            if (defined('KOE_CAPABILITY_GAB') && KOE_CAPABILITY_GAB)                    $cap[] = "gab";
+            if (defined('KOE_CAPABILITY_RECEIVEFLAGS') && KOE_CAPABILITY_RECEIVEFLAGS)  $cap[] = "receiveflags";
+            if (defined('KOE_CAPABILITY_SENDFLAGS') && KOE_CAPABILITY_SENDFLAGS)        $cap[] = "sendflags";
+            if (defined('KOE_CAPABILITY_OOFTIMES') && KOE_CAPABILITY_OOFTIMES)          $cap[] = "ooftime";
+            elseif(defined('KOE_CAPABILITY_OOF') && KOE_CAPABILITY_OOF)                 $cap[] = "oof";        // 'ooftime' superseeds 'oof'. If 'ooftime' is set, 'oof' should not be defined.
+            if (defined('KOE_CAPABILITY_NOTES') && KOE_CAPABILITY_NOTES)                $cap[] = "notes";
+            if (defined('KOE_CAPABILITY_SHAREDFOLDER') && KOE_CAPABILITY_SHAREDFOLDER)  $cap[] = "sharedfolder";
+            if (defined('KOE_CAPABILITY_SENDAS') && KOE_CAPABILITY_SENDAS)              $cap[] = "sendas";
+
+            self::$specialHeaders = array();
+            self::$specialHeaders[] = "X-Push-Capabilities: ". implode(",",$cap);
+            self::$specialHeaders[] = "X-Push-GAB-Name: ". bin2hex(KOE_GAB_NAME);
+        }
+
         //save the request parameters
         $request = array();
 
@@ -64,7 +82,8 @@ class Settings extends RequestProcessor {
         // - DeviceInformation
         // - UserInformation
         // Each of them should only be once per request. Each property must be processed in order.
-        while (1) {
+        WBXMLDecoder::ResetInWhile("settingsMain");
+        while(WBXMLDecoder::InWhile("settingsMain")) {
             $propertyName = "";
             if (self::$decoder->getElementStartTag(SYNC_SETTINGS_OOF)) {
                 $propertyName = SYNC_SETTINGS_OOF;
@@ -229,4 +248,3 @@ class Settings extends RequestProcessor {
         return true;
     }
 }
-?>
\ No newline at end of file
