@@ -10,7 +10,7 @@
 *
 * Created   :   05.09.2011
 *
-* Copyright 2007 - 2013 Zarafa Deutschland GmbH
+* Copyright 2007 - 2016 Zarafa Deutschland GmbH
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
@@ -86,6 +86,7 @@ class SyncMail extends SyncObject {
     public $lastverbexectime;
     public $receivedasbcc;
     public $sender;
+    public $categories;
 
     function SyncMail() {
         $mapping = array (
@@ -101,12 +102,16 @@ class SyncMail extends SyncObject {
 
                     SYNC_POOMMAIL_FROM                                  => array (  self::STREAMER_VAR      => "from",
                                                                                     self::STREAMER_CHECKS   => array(   self::STREAMER_CHECK_LENGTHMAX      => 32768,
-                                                                                                                        self::STREAMER_CHECK_EMAIL        => "" )),
+                                                                                                                        self::STREAMER_CHECK_EMAIL        => "" ),
+                                                                                    self::STREAMER_RONOTIFY => true),
 
-                    SYNC_POOMMAIL_SUBJECT                               => array (  self::STREAMER_VAR      => "subject"),
-                    SYNC_POOMMAIL_THREADTOPIC                           => array (  self::STREAMER_VAR      => "threadtopic"),
+                    SYNC_POOMMAIL_SUBJECT                               => array (  self::STREAMER_VAR      => "subject",
+                                                                                    self::STREAMER_RONOTIFY => true),
+                    SYNC_POOMMAIL_THREADTOPIC                           => array (  self::STREAMER_VAR      => "threadtopic",
+                                                                                    self::STREAMER_RONOTIFY => true),
                     SYNC_POOMMAIL_DATERECEIVED                          => array (  self::STREAMER_VAR      => "datereceived",
-                                                                                    self::STREAMER_TYPE     => self::STREAMER_TYPE_DATE_DASHES),
+                                                                                    self::STREAMER_TYPE     => self::STREAMER_TYPE_DATE_DASHES,
+                                                                                    self::STREAMER_RONOTIFY => true),
 
                     SYNC_POOMMAIL_DISPLAYTO                             => array (  self::STREAMER_VAR      => "displayto"),
 
@@ -120,7 +125,10 @@ class SyncMail extends SyncObject {
                                                                                                                         self::STREAMER_CHECK_ONEVALUEOF     => array(0,1,2) )),
 
                     SYNC_POOMMAIL_READ                                  => array (  self::STREAMER_VAR      => "read",
-                                                                                    self::STREAMER_CHECKS   => array(   self::STREAMER_CHECK_ONEVALUEOF     => array(0,1) )),
+                                                                                    self::STREAMER_CHECKS   => array(   self::STREAMER_CHECK_ONEVALUEOF     => array(0,1) ),
+                                                                                    self::STREAMER_RONOTIFY => true,
+                                                                                    self::STREAMER_VALUEMAP => array(   0 => "No",
+                                                                                                                        1 => "Yes")),
 
                     SYNC_POOMMAIL_ATTACHMENTS                           => array (  self::STREAMER_VAR      => "attachments",
                                                                                     self::STREAMER_TYPE     => "SyncAttachment",
@@ -129,7 +137,8 @@ class SyncMail extends SyncObject {
                     SYNC_POOMMAIL_MIMETRUNCATED                         => array (  self::STREAMER_VAR      => "mimetruncated",
                                                                                     self::STREAMER_CHECKS   => array(   self::STREAMER_CHECK_ZEROORONE      => self::STREAMER_CHECK_SETZERO)),
 
-                    SYNC_POOMMAIL_MIMEDATA                              => array (  self::STREAMER_VAR      => "mimedata"), //TODO mimedata should be of a type stream
+                    SYNC_POOMMAIL_MIMEDATA                              => array (  self::STREAMER_VAR      => "mimedata",
+                                                                                    self::STREAMER_TYPE     => self::STREAMER_TYPE_STREAM_ASPLAIN),
 
                     SYNC_POOMMAIL_MIMESIZE                              => array (  self::STREAMER_VAR      => "mimesize",
                                                                                     self::STREAMER_CHECKS   => array(   self::STREAMER_CHECK_CMPHIGHER      => -1)),
@@ -168,7 +177,8 @@ class SyncMail extends SyncObject {
 
             $mapping[SYNC_POOMMAIL_FLAG]                                = array (   self::STREAMER_VAR      => "flag",
                                                                                     self::STREAMER_TYPE     => "SyncMailFlags",
-                                                                                    self::STREAMER_PROP     => self::STREAMER_TYPE_SEND_EMPTY);
+                                                                                    self::STREAMER_PROP     => self::STREAMER_TYPE_SEND_EMPTY,
+                                                                                    self::STREAMER_RONOTIFY => true);
 
             $mapping[SYNC_AIRSYNCBASE_NATIVEBODYTYPE]                   = array (   self::STREAMER_VAR      => "nativebodytype");
 
@@ -189,12 +199,11 @@ class SyncMail extends SyncObject {
             $mapping[SYNC_POOMMAIL2_RECEIVEDASBCC]                      = array (   self::STREAMER_VAR      => "receivedasbcc");
             $mapping[SYNC_POOMMAIL2_SENDER]                             = array (   self::STREAMER_VAR      => "sender");
             $mapping[SYNC_POOMMAIL_CATEGORIES]                          = array (   self::STREAMER_VAR      => "categories",
-                                                                                    self::STREAMER_ARRAY    => SYNC_POOMMAIL_CATEGORY);
+                                                                                    self::STREAMER_ARRAY    => SYNC_POOMMAIL_CATEGORY,
+                                                                                    self::STREAMER_RONOTIFY => true);
             //TODO bodypart, accountid, rightsmanagementlicense
         }
 
         parent::SyncObject($mapping);
     }
 }
-
-?>
\ No newline at end of file
