@@ -6,7 +6,7 @@
 *
 * Created   :   01.10.2007
 *
-* Copyright 2007 - 2012 Zarafa Deutschland GmbH
+* Copyright 2007 - 2013 Zarafa Deutschland GmbH
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
@@ -42,26 +42,7 @@
 ************************************************/
 
 
-define('WBXML_SWITCH_PAGE',     0x00);
-define('WBXML_END',             0x01);
-define('WBXML_ENTITY',          0x02);
-define('WBXML_STR_I',           0x03);
-define('WBXML_LITERAL',         0x04);
-define('WBXML_EXT_I_0',         0x40);
-define('WBXML_EXT_I_1',         0x41);
-define('WBXML_EXT_I_2',         0x42);
-define('WBXML_PI',              0x43);
-define('WBXML_LITERAL_C',       0x44);
-define('WBXML_EXT_T_0',         0x80);
-define('WBXML_EXT_T_1',         0x81);
-define('WBXML_EXT_T_2',         0x82);
-define('WBXML_STR_T',           0x83);
-define('WBXML_LITERAL_A',       0x84);
-define('WBXML_EXT_0',           0xC0);
-define('WBXML_EXT_1',           0xC1);
-define('WBXML_EXT_2',           0xC2);
-define('WBXML_OPAQUE',          0xC3);
-define('WBXML_LITERAL_AC',      0xC4);
+
 
 define('EN_TYPE',               1);
 define('EN_TAG',                2);
@@ -77,6 +58,31 @@ define('EN_FLAGS_CONTENT',      1);
 define('EN_FLAGS_ATTRIBUTES',   2);
 
 class WBXMLDefs {
+
+    const WBXML_SWITCH_PAGE = 0x00;
+    const WBXML_END =         0x01;
+    const WBXML_ENTITY =      0x02; //not used in ActiveSync
+    const WBXML_STR_I =       0x03;
+    const WBXML_LITERAL =     0x04; //not used in ActiveSync
+    const WBXML_EXT_I_0 =     0x40; //not used in ActiveSync
+    const WBXML_EXT_I_1 =     0x41; //not used in ActiveSync
+    const WBXML_EXT_I_2 =     0x42; //not used in ActiveSync
+    const WBXML_PI =          0x43; //not used in ActiveSync
+    const WBXML_LITERAL_C =   0x44; //not used in ActiveSync
+    const WBXML_EXT_T_0 =     0x80; //not used in ActiveSync
+    const WBXML_EXT_T_1 =     0x81; //not used in ActiveSync
+    const WBXML_EXT_T_2 =     0x82; //not used in ActiveSync
+    const WBXML_STR_T =       0x83; //not used in ActiveSync
+    const WBXML_LITERAL_A =   0x84; //not used in ActiveSync
+    const WBXML_EXT_0 =       0xC0; //not used in ActiveSync
+    const WBXML_EXT_1 =       0xC1; //not used in ActiveSync
+    const WBXML_EXT_2 =       0xC2; //not used in ActiveSync
+    const WBXML_OPAQUE =      0xC3;
+    const WBXML_LITERAL_AC =  0xC4; //not used in ActiveSync
+
+    const WBXML_WITH_ATTRIBUTES = 0x80; //not used in ActiveSync
+    const WBXML_WITH_CONTENT = 0x40;
+
     /**
      * The WBXML DTDs
      */
@@ -748,7 +754,7 @@ class WBXMLDefs {
                   8 => "MeetingResponse",
                   9 => "POOMTASKS",
                   0xA => "ResolveRecipients",
-                  0xB => "ValidateCerts",
+                  0xB => "ValidateCert",
                   0xC => "POOMCONTACTS2",
                   0xD => "Ping",
                   0xE => "Provision",//
@@ -765,5 +771,3 @@ class WBXMLDefs {
               )
           );
 }
-
-?>
\ No newline at end of file
