@@ -1,11 +1,11 @@
-#!/usr/bin/php
+#!/usr/bin/env php
 <?php
 /***********************************************
 * File      :   z-push-admin.php
 * Project   :   Z-Push
 * Descr     :   This is a small command line
 *               client to see and modify the
-*               wipe status of Zarafa users.
+*               wipe status of Kopano users.
 *
 * Created   :   14.05.2010
 *
@@ -44,57 +44,10 @@
 * Consult LICENSE file for details
 ************************************************/
 
-include('lib/core/zpushdefs.php');
-include('lib/core/zpush.php');
-include('lib/core/stateobject.php');
-include('lib/core/syncparameters.php');
-include('lib/core/bodypreference.php');
-include('lib/core/contentparameters.php');
-include('lib/core/synccollections.php');
-include('lib/core/zlog.php');
-include('lib/core/statemanager.php');
-include('lib/core/streamer.php');
-include('lib/core/asdevice.php');
-include('lib/core/interprocessdata.php');
-include('lib/core/loopdetection.php');
-include('lib/exceptions/exceptions.php');
-include('lib/utils/utils.php');
-include('lib/utils/zpushadmin.php');
-include('lib/request/request.php');
-include('lib/request/requestprocessor.php');
-include('lib/interface/ibackend.php');
-include('lib/interface/ichanges.php');
-include('lib/interface/iexportchanges.php');
-include('lib/interface/iimportchanges.php');
-include('lib/interface/isearchprovider.php');
-include('lib/interface/istatemachine.php');
-include('lib/syncobjects/syncobject.php');
-include('lib/syncobjects/syncbasebody.php');
-include('lib/syncobjects/syncbaseattachment.php');
-include('lib/syncobjects/syncmailflags.php');
-include('lib/syncobjects/syncrecurrence.php');
-include('lib/syncobjects/syncappointment.php');
-include('lib/syncobjects/syncappointmentexception.php');
-include('lib/syncobjects/syncattachment.php');
-include('lib/syncobjects/syncattendee.php');
-include('lib/syncobjects/syncmeetingrequestrecurrence.php');
-include('lib/syncobjects/syncmeetingrequest.php');
-include('lib/syncobjects/syncmail.php');
-include('lib/syncobjects/syncnote.php');
-include('lib/syncobjects/synccontact.php');
-include('lib/syncobjects/syncfolder.php');
-include('lib/syncobjects/syncprovisioning.php');
-include('lib/syncobjects/synctaskrecurrence.php');
-include('lib/syncobjects/synctask.php');
-include('lib/syncobjects/syncoofmessage.php');
-include('lib/syncobjects/syncoof.php');
-include('lib/syncobjects/syncuserinformation.php');
-include('lib/syncobjects/syncdeviceinformation.php');
-include('lib/syncobjects/syncdevicepassword.php');
-include('lib/syncobjects/syncitemoperationsattachment.php');
+require_once 'vendor/autoload.php';
+
 if (!defined('ZPUSH_CONFIG')) define('ZPUSH_CONFIG', 'config.php');
 include_once(ZPUSH_CONFIG);
-include('version.php');
 
 /**
  * //TODO resync of single folders of a users device
@@ -122,7 +75,8 @@ include('version.php');
         ZPushAdminCLI::RunCommand();
     }
     catch (ZPushException $zpe) {
-        die(get_class($zpe) . ": ". $zpe->getMessage() . "\n");
+        fwrite(STDERR, get_class($zpe) . ": ". $zpe->getMessage() . "\n");
+        exit(1);
     }
 
 
@@ -148,6 +102,7 @@ class ZPushAdminCLI {
     const TYPE_OPTION_TASK = "task";
     const TYPE_OPTION_NOTE = "note";
     const TYPE_OPTION_HIERARCHY = "hierarchy";
+    const TYPE_OPTION_GAB = "gab";
 
     static private $command;
     static private $user = false;
@@ -162,9 +117,9 @@ class ZPushAdminCLI {
      * @access public
      */
     static public function UsageInstructions() {
-        $types = "'".self::TYPE_OPTION_EMAIL."', '".self::TYPE_OPTION_CALENDAR."', '".self::TYPE_OPTION_CONTACT."', '".self::TYPE_OPTION_TASK."' or '".self::TYPE_OPTION_NOTE."'";
         return  "Usage:\n\tz-push-admin.php -a ACTION [options]\n\n" .
-                "Parameters:\n\t-a list/wipe/remove/resync/clearloop\n\t[-u] username\n\t[-d] deviceid\n\n" .
+                "Parameters:\n\t-a list/lastsync/wipe/remove/resync/clearloop/fixstates\n\t[-u] username\n\t[-d] deviceid\n" .
+                "\t[-t] type\tthe following types are available: '".self::TYPE_OPTION_EMAIL."', '".self::TYPE_OPTION_CALENDAR."', '".self::TYPE_OPTION_CONTACT."', '".self::TYPE_OPTION_TASK."', '".self::TYPE_OPTION_NOTE."', '".self::TYPE_OPTION_HIERARCHY."' of '".self::TYPE_OPTION_GAB."' (for KOE) or a folder id.\n\n" .
                 "Actions:\n" .
                 "\tlist\t\t\t\t\t Lists all devices and synchronized users\n" .
                 "\tlist -u USER\t\t\t\t Lists all devices of user USER\n" .
@@ -177,9 +132,9 @@ class ZPushAdminCLI {
                 "\tremove -d DEVICE\t\t\t Removes all state data of all users synchronized on device DEVICE\n" .
                 "\tremove -u USER -d DEVICE\t\t Removes all related state data of device DEVICE of user USER\n" .
                 "\tresync -u USER -d DEVICE\t\t Resynchronizes all data of device DEVICE of user USER\n" .
-                "\tresync -t TYPE \t\t\t\t Resynchronizes all folders of type $types for all devices and users.\n" .
-                "\tresync -t TYPE -u USER \t\t\t Resynchronizes all folders of type $types for the user USER.\n" .
-                "\tresync -t TYPE -u USER -d DEVICE\t Resynchronizes all folders of type $types for a specified device and user.\n" .
+                "\tresync -t TYPE \t\t\t\t Resynchronizes all folders of type (possible values above) for all devices and users.\n" .
+                "\tresync -t TYPE -u USER \t\t\t Resynchronizes all folders of type (possible values above) for the user USER.\n" .
+                "\tresync -t TYPE -u USER -d DEVICE\t Resynchronizes all folders of type (possible values above) for a specified device and user.\n" .
                 "\tresync -t FOLDERID -u USER\t\t Resynchronize the specified folder id only. The USER should be specified for better performance.\n" .
                 "\tresync -t hierarchy -u USER -d DEVICE\t Resynchronize the folder hierarchy data for an optional USER and optional DEVICE.\n" .
                 "\tclearloop\t\t\t\t Clears system wide loop detection data\n" .
@@ -239,7 +194,7 @@ class ZPushAdminCLI {
         elseif (isset($options['type']) && !empty($options['type']))
             self::$type = strtolower(trim($options['type']));
 
-        // if type is set, it must be one of known types or a 44 byte long folder id
+        // if type is set, it must be one of known types or a 44 or 48 byte long folder id
         if (self::$type !== false) {
             if (self::$type !== self::TYPE_OPTION_EMAIL &&
                 self::$type !== self::TYPE_OPTION_CALENDAR &&
@@ -247,10 +202,13 @@ class ZPushAdminCLI {
                 self::$type !== self::TYPE_OPTION_TASK &&
                 self::$type !== self::TYPE_OPTION_NOTE &&
                 self::$type !== self::TYPE_OPTION_HIERARCHY &&
-                strlen(self::$type) !== 44) {
+                self::$type !== self::TYPE_OPTION_GAB &&
+                strlen(self::$type) !== 6 &&       // like U1f38d
+                strlen(self::$type) !== 44 &&
+                strlen(self::$type) !== 48) {
                     self::$errormessage = "Wrong 'type'. Possible values are: ".
-                        "'".self::TYPE_OPTION_EMAIL."', '".self::TYPE_OPTION_CALENDAR."', '".self::TYPE_OPTION_CONTACT."', '".self::TYPE_OPTION_TASK."', '".self::TYPE_OPTION_NOTE."', ".self::TYPE_OPTION_HIERARCHY."' ".
-                        "or a 44 byte long folder id (as hex).";
+                        "'".self::TYPE_OPTION_EMAIL."', '".self::TYPE_OPTION_CALENDAR."', '".self::TYPE_OPTION_CONTACT."', '".self::TYPE_OPTION_TASK."', '".self::TYPE_OPTION_NOTE."', '".self::TYPE_OPTION_HIERARCHY."', '".self::TYPE_OPTION_GAB."' ".
+                        "or a 6, 44 or 48 byte long folder id (as hex).";
                     return;
                 }
         }
@@ -484,7 +442,7 @@ class ZPushAdminCLI {
             echo "\tno devices found\n";
         else {
             echo "All known devices and users and their last synchronization time\n\n";
-            echo str_pad("Device id", 36). str_pad("Synchronized user", 31)."Last sync time\n";
+            echo str_pad("Device id", 36) . str_pad("Synchronized user", 31) . str_pad("Last sync time", 20) . "Short Ids\n";
             echo "-----------------------------------------------------------------------------------------------------\n";
         }
 
@@ -492,7 +450,9 @@ class ZPushAdminCLI {
             $users = ZPushAdmin::ListUsers($deviceId);
             foreach ($users as $user) {
                 $device = ZPushAdmin::GetDeviceDetails($deviceId, $user);
-                echo str_pad($deviceId, 36) . str_pad($user, 30) . " " . ($device->GetLastSyncTime() ? strftime("%Y-%m-%d %H:%M", $device->GetLastSyncTime()) : "never") . "\n";
+                $lastsync = $device->GetLastSyncTime() ? strftime("%Y-%m-%d %H:%M", $device->GetLastSyncTime()) : "never";
+                $hasShortFolderIds = $device->HasFolderIdMapping() ? "Yes":"No";
+                echo str_pad($deviceId, 36) . str_pad($user, 30) . " " . str_pad($lastsync, 20) . $hasShortFolderIds . "\n";
             }
         }
     }
@@ -676,9 +636,27 @@ class ZPushAdminCLI {
         }
 
         $folders = array();
+        $searchFor = $type;
+        // get the KOE gab folderid
+        if ($type == self::TYPE_OPTION_GAB) {
+            if (@constant('KOE_GAB_FOLDERID') !== '') {
+                $gab = KOE_GAB_FOLDERID;
+            }
+            else {
+                $gab = $device->GetKoeGabBackendFolderId();
+            }
+            if (!$gab) {
+                printf("Could not find KOE GAB folderid for device '%s' of user '%s'\n", $deviceId, $user);
+                return false;
+            }
+            $searchFor = $gab;
+        }
+        // potential long ids are converted to folderids here, incl. the gab id
+        $searchFor = strtolower($device->GetFolderIdForBackendId($searchFor, false, false, null));
+
         foreach ($device->GetAllFolderIds() as $folderid) {
             // if  submitting a folderid as type to resync a specific folder.
-            if ($folderid == $type) {
+            if (strtolower($folderid) === $searchFor) {
                 printf("Found and resynching requested folderid '%s' on device '%s' of user '%s'\n", $folderid, $deviceId, $user);
                 $folders[] = $folderid;
                 break;
@@ -689,26 +667,26 @@ class ZPushAdminCLI {
                 switch($foldertype) {
                     case SYNC_FOLDER_TYPE_APPOINTMENT:
                     case SYNC_FOLDER_TYPE_USER_APPOINTMENT:
-                        if ($type == "calendar")
+                        if ($searchFor == "calendar")
                             $folders[] = $folderid;
                         break;
                     case SYNC_FOLDER_TYPE_CONTACT:
                     case SYNC_FOLDER_TYPE_USER_CONTACT:
-                        if ($type == "contact")
+                        if ($searchFor == "contact")
                             $folders[] = $folderid;
                         break;
                     case SYNC_FOLDER_TYPE_TASK:
                     case SYNC_FOLDER_TYPE_USER_TASK:
-                        if ($type == "task")
+                        if ($searchFor == "task")
                             $folders[] = $folderid;
                         break;
                     case SYNC_FOLDER_TYPE_NOTE:
                     case SYNC_FOLDER_TYPE_USER_NOTE:
-                        if ($type == "note")
+                        if ($searchFor == "note")
                             $folders[] = $folderid;
                         break;
                     default:
-                        if ($type == "email")
+                        if ($searchFor == "email")
                             $folders[] = $folderid;
                         break;
                 }
@@ -716,7 +694,7 @@ class ZPushAdminCLI {
         }
 
         $stat = ZPushAdmin::ResyncFolder($user, $deviceId, $folders);
-        echo sprintf("Resync of %d folders of type %s on device '%s' of user '%s': %s\n", count($folders), $type, $deviceId, $user, ($stat)?'Requested':ZLog::GetLastMessage(LOGLEVEL_ERROR));
+        echo sprintf("Resync of %d folders of type '%s' on device '%s' of user '%s': %s\n", count($folders), $type, $deviceId, $user, ($stat)?'Requested':ZLog::GetLastMessage(LOGLEVEL_ERROR));
     }
 
     /**
@@ -760,6 +738,18 @@ class ZPushAdminCLI {
             printf("Processed: %d - Deleted: %d\n",  $stat[0], $stat[1]);
         else
             echo ZLog::GetLastMessage(LOGLEVEL_ERROR) . "\n";
+
+        echo "\tChecking for hierarchy folder data state: ";
+        if (($stat = ZPushAdmin::FixStatesHierarchyFolderData()) !== false)
+            printf("Devices: %d - Processed: %d - Fixed: %d - Device+User without hierarchy: %d\n",  $stat[0], $stat[1], $stat[2], $stat[3]);
+        else
+            echo ZLog::GetLastMessage(LOGLEVEL_ERROR) . "\n";
+
+        echo "\tChecking flags of shared folders: ";
+        if (($stat = ZPushAdmin::FixStatesAdditionalFolderFlags()) !== false)
+            printf("Devices: %d - Devices with additional folders: %d - Fixed: %d\n",  $stat[0], $stat[1], $stat[2]);
+        else
+            echo ZLog::GetLastMessage(LOGLEVEL_ERROR) . "\n";
     }
 
     /**
@@ -788,10 +778,17 @@ class ZPushAdminCLI {
             if ($device->GetFolderUUID($folderid)) {
                 $synchedFolders++;
                 $type = $device->GetFolderType($folderid);
+                $folder = $device->GetHierarchyCache()->GetFolder($folderid);
+                $name = $folder ? $folder->displayname : "unknown";
                 switch($type) {
                     case SYNC_FOLDER_TYPE_APPOINTMENT:
                     case SYNC_FOLDER_TYPE_USER_APPOINTMENT:
-                        $gentype = "Calendars";
+                        if (KOE_GAB_NAME != "" && $name == KOE_GAB_NAME) {
+                            $gentype = "GAB";
+                        }
+                        else {
+                            $gentype = "Calendars";
+                        }
                         break;
                     case SYNC_FOLDER_TYPE_CONTACT:
                     case SYNC_FOLDER_TYPE_USER_CONTACT:
@@ -816,8 +813,7 @@ class ZPushAdminCLI {
                 // set the folder name for all folders which are not fully synchronized yet
                 $fstatus = $device->GetFolderSyncStatus($folderid);
                 if ($fstatus !== false && is_array($fstatus)) {
-                    // TODO would be nice if we could see the real name of the folder, right now we use the folder type as name
-                    $fstatus['name'] = $gentype;
+                    $fstatus['name'] = $name ? $name : $gentype;
                     $device->SetFolderSyncStatus($folderid, $fstatus);
                     $syncedFoldersInProgress++;
                 }
@@ -859,6 +855,7 @@ class ZPushAdminCLI {
         echo "First sync:\t\t". strftime("%Y-%m-%d %H:%M", $device->GetFirstSyncTime()) ."\n";
         echo "Last sync:\t\t". ($device->GetLastSyncTime() ? strftime("%Y-%m-%d %H:%M", $device->GetLastSyncTime()) : "never")."\n";
         echo "Total folders:\t\t". count($folders). "\n";
+        echo "Short folder Ids:\t". ($device->HasFolderIdMapping() ? "Yes":"No") ."\n";
         echo "Synchronized folders:\t". $synchedFolders;
         if ($syncedFoldersInProgress > 0)
             echo " (". $syncedFoldersInProgress. " in progress)";
@@ -874,7 +871,10 @@ class ZPushAdminCLI {
                         $percent = round($d['done']*100/$d['total']);
                         $status = sprintf("Status: %s%d%% (%d/%d)", ($percent < 10)?" ":"", $percent, $d['done'], $d['total']);
                     }
-                    printf("\tFolder: %s%s Sync: %s    %s\n", $d['name'], str_repeat(" ", 12-strlen($d['name'])), $d['status'], $status);
+                    if (strlen($d['name']) > 20) {
+                        $d['name'] = substr($d['name'], 0, 18) . "..";
+                    }
+                    printf("\tFolder: %s Sync: %s %s\n", str_pad($d['name'], 20), str_pad($d['status'], 13), $status);
                 }
             }
         }
@@ -900,6 +900,14 @@ class ZPushAdminCLI {
         echo "WipeRequest on:\t\t". ($device->GetWipeRequestedOn() ? strftime("%Y-%m-%d %H:%M", $device->GetWipeRequestedOn()) : "not set")."\n";
         echo "WipeRequest by:\t\t". ($device->GetWipeRequestedBy() ? $device->GetWipeRequestedBy() : "not set")."\n";
         echo "Wiped on:\t\t". ($device->GetWipeActionOn() ? strftime("%Y-%m-%d %H:%M", $device->GetWipeActionOn()) : "not set")."\n";
+        echo "Policy name:\t\t". ($device->GetPolicyName() ? $device->GetPolicyName() : ASDevice::DEFAULTPOLICYNAME)."\n";
+
+        if ($device->GetKoeVersion()) {
+            echo "Kopano Outlook Extension:\n";
+            echo "\tVersion:\t". $device->GetKoeVersion() ."\n";
+            echo "\tBuild:\t\t". $device->GetKoeBuild() ."\n";
+            echo "\tBuild Date:\t". strftime("%Y-%m-%d %H:%M",$device->GetKoeBuildDate()) ."\n";
+        }
 
         echo "Attention needed:\t";
 
@@ -933,6 +941,3 @@ class ZPushAdminCLI {
 
     }
 }
-
-
-?>
\ No newline at end of file
